
import 'package:flutter/material.dart';
import 'calendar_page_viewModel.dart';
import 'calendar_widget.dart';
import 'time_utils.dart';

enum Location{left,mid,right}

typedef void SelectDateOnTap(DayModel startTimeModel, DayModel endTimeModel);

/// 选择日期提示弹框
class CalendarPageDialog extends StatefulWidget {
  final DayModel startTimeModel;// 外部传入的之前选中的开始日期
  final DayModel endTimeModel;// 外部传入的之前选中的结束日期
  final SelectDateOnTap selectDateOnTap;// 确定按钮的callback 给外部传值
  final Function onPressedCancel; //取消
  CalendarPageDialog({this.startTimeModel,this.endTimeModel,this.selectDateOnTap, this.onPressedCancel});

  @override
  CalendarPageDialogState createState() => CalendarPageDialogState();
}

class CalendarPageDialogState extends State<CalendarPageDialog> with
    AutomaticKeepAliveClientMixin<CalendarPageDialog>{

  String _selectCheckInTime = '选择开始时间';
  String _selectLeaveTime = '选择结束时间';
  bool _isSelectCheckInTime = false; // 是否选择开始日期
  bool _isSelectLeaveTime = false; // 是否选择结束日期
  int _checkInDays = 0; // 天数

  // 保存当前选中的开始日期和结束日期
  DayModel _selectCheckInTimeModel = null;
  DayModel _isSelectLeaveTimeModel = null;

  ScrollController _listViewController = ScrollController();

  List<CalendarItemViewModel> _list = [];

  @override
  // TODO: implement wantKeepAlive
  bool get wantKeepAlive => true;


  @override
  void initState() {
    super.initState();
    // 加载日历数据源
    _list = CalendarViewModel().getItemList();

    double scrollY = -1;
    Future.delayed(Duration(milliseconds: 30), (){
      for(int i=0; i < _list.length; i++){
        CalendarItemViewModel mm = _list[i];
        int rows = TimeUtil.getRowsForMonthYear(mm.year,
            mm.month, MaterialLocalizations.of(context));
        mm.heightTemp =  rows * 54.0 + 20.0 + 20.0 + 25.0;

        if(widget.startTimeModel!=null && widget.endTimeModel!=null) {
          if(widget.startTimeModel.year >= mm.year && widget.startTimeModel.month >= mm.month){
            scrollY = scrollY+mm.heightTemp;
          }
        }
      }

      if (scrollY > 0) {
        _listViewController.animateTo(scrollY,
            duration: new Duration(microseconds: 300), curve: Curves.ease);
      } else {
        _listViewController.animateTo(10000,
            duration: new Duration(microseconds: 300), curve: Curves.ease);
      }

    });

    // 处理外部传入的选中日期
    if(widget.startTimeModel!=null && widget.endTimeModel!=null) {
      for(int i=0; i<_list.length; i++) {
        CalendarItemViewModel model = _list[i];
        if(model.month == widget.startTimeModel.month) {
          _updateDataSource(widget.startTimeModel.year, widget.startTimeModel.month, widget.startTimeModel.dayNum);
        }
        if (model.month == widget.endTimeModel.month) {
          _updateDataSource(widget.endTimeModel.year, widget.endTimeModel.month, widget.endTimeModel.dayNum);
        }
      }
    }

  }


    @override
  Widget build(BuildContext context) {
    // TODO: implement build

      var bottomButton = Container(
      width: MediaQuery
          .of(context)
          .size
          .width,
      height: 70,
      color: Color(0xFFFFFFFF),
      child: Container(
        margin: EdgeInsets.only(left: 15, right: 15, top: 10, bottom: 10),
        decoration: BoxDecoration(
          color: Color(0xFF2E81F8),
          borderRadius: BorderRadius.all(Radius.circular(5)),
        ),
        child: new FlatButton(
          splashColor: Colors.transparent,
          highlightColor: Colors.transparent,
          padding: EdgeInsets.all(0),
          onPressed: () {
            _sureButtonTap();
          },
          child: Container(
            alignment: Alignment.center,
            child: Text(
              "完成",
              style: TextStyle(color: Colors.white, fontSize: 18),
            ),
          ),
        ),
      ),
    );

    var contentWidget = Container(
      height: 400,
      color: Color(0x00000000),
      child: Column(
        children: [
          _weekItem(MediaQuery.of(context).size.width), //30
          // 月日期的视图
          Container(
            height: 400-30.0,
            child: ListView.builder(
              itemBuilder: (BuildContext context, int index) {
                if(index < _list.length){
                  CalendarItemViewModel itemModel = _list[index];
                  return CalendarItem(
                        (year, month, checkInTime) {
                      _updateCheckInLeaveTime(
                          year, month, checkInTime);
                    },
                    itemModel,
                  );
                }else{
                  return Container(
                    height: 20,
                  );
                }
              },
              itemCount: _list.length+1,
              controller: _listViewController,
            ),
          ),
        ],
      ),
    );

    var topWidget = Container(
      width: MediaQuery
          .of(context)
          .size
          .width,
      height: 60,
      color: Color(0xFFFFFFFF),
      child: Row(
        children: [
          GestureDetector(
            child: Container(
              child: Image.asset('images/close.png'),
              margin: EdgeInsets.only(left: 15),
              width: 20,
              height: 20,
            ),
            onTap: (){
              Navigator.pop(context);
            },
          ),
          Container(
            alignment: Alignment.center,
            width: MediaQuery.of(context).size.width-70,
            child: Text(
              "选择日期",
              style: TextStyle(color: Color(0xFF333333), fontSize: 18),
            ),
          ),
        ],
      ),
    );

    var body = Material(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          topWidget,
          contentWidget,
          bottomButton
        ],
      ),
    );

    return AnimatedPadding(
      padding: MediaQuery
          .of(context)
          .viewInsets +
          const EdgeInsets.symmetric(horizontal: 0.0, vertical: 0.0),
      duration: const Duration(milliseconds: 120),
      curve: Curves.easeInCubic,
      child: MediaQuery.removeViewInsets(
        removeLeft: true,
        removeTop: true,
        removeRight: true,
        removeBottom: true,
        context: context,
        child: SizedBox(
          width: double.infinity,
          child: body,
        ),
      ),
    );
  }


  /*
  * 显示周的组件，使用了 _weekTitleItem
  * */
  _weekItem(double screenW) {
    List<String> _listS = <String>[
      '日',
      '一',
      '二',
      '三',
      '四',
      '五',
      '六',
    ];
    List<Widget> _listW = [];
    _listS.forEach((title) {
      _listW.add(_weekTitleItem(title, (screenW - 40) / 7));
    });

    return Container(
      height: 30,
      width: screenW,
      color: Colors.white,
      child: Container(
        margin: EdgeInsets.only(left: 20, right: 20),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: _listW,
        ),
      ),
    );
  }

  /*
  * 周内对应的每天的组件
  * */
  _weekTitleItem(String title, double width) {
    return Container(
      alignment: Alignment.center,
      width: width,
      child: Text(
        title,
        style: TextStyle(
          color: Color(0xFF333333),
          fontSize: 14,
          fontFamily: 'PingFangSC-Semibold',
          fontWeight: FontWeight.bold
        ),
      ),
    );
  }

  /*
  * 比较后面的日期是比model日期小（left） 还是相等(mid) 还是大 (right)
  * */
  _comparerTime(DayModel model, int year, int month, int day){
    if(year > model.year) {
      return Location.right;
    } else if(year == model.year) {
      if((model.month+model.year*100) < (month+year*100)) {
        return Location.right;
      } else if((model.month+model.year*100) == (month+year*100)){
        if(model.dayNum < day){
          return Location.right;
        } else if(model.dayNum == day){
          return Location.mid;
        } else {
          return Location.left;
        }
      } else {
        return Location.left;
      }
    } else {
      return Location.left;
    }
  }

  /*
  * 更新日历的数据源
  * */
  _updateDataSource(int year, int month, int checkInTime) {
    // 左右指针 用来记录选择的入住日期和离开日期
    DayModel firstModel = null ;
    DayModel lastModel = null;

    for(int i=0; i<_list.length; i++) {
      CalendarItemViewModel model = _list[i];
      if(model.firstSelectModel != null){
        firstModel = model.firstSelectModel;
      }
      if (model.lastSelectModel != null) {
        lastModel = model.lastSelectModel;
      }
    }

    if (firstModel != null && lastModel != null) {
      for(int i=0; i<_list.length; i++) {
        CalendarItemViewModel model = _list[i];
        model.firstSelectModel = null;
        model.lastSelectModel = null;
        firstModel = null;
        lastModel = null;
        for(int i=0; i<model.list.length; i++) {
          DayModel dayModel = model.list[i];
          dayModel.isSelect = false;
          if(_comparerTime(dayModel, year, month, checkInTime) == Location.mid){
            dayModel.isSelect = true;
            model.firstSelectModel = dayModel;
            _isSelectCheckInTime = true;
            _isSelectLeaveTime = false;
            _selectCheckInTime = '$year.$month.$checkInTime';
            _selectCheckInTimeModel = dayModel;
          }
        }
      }
      _checkInDays = 0;
      _isSelectLeaveTime = false;
      _selectLeaveTime = '选择离开时间';
      _isSelectLeaveTimeModel = null;
    } else if(firstModel != null && lastModel == null) {
      if(_comparerTime(firstModel, year, month, checkInTime) == Location.left){
        for(int i=0; i<_list.length; i++) {
          CalendarItemViewModel model = _list[i];
          model.firstSelectModel = null;
          model.lastSelectModel = null;
          firstModel = null;
          lastModel = null;
          for(int i=0; i<model.list.length; i++) {
            DayModel dayModel = model.list[i];
            dayModel.isSelect = false;
            if(_comparerTime(dayModel, year, month, checkInTime) == Location.mid){
              dayModel.isSelect = !dayModel.isSelect;
              model.firstSelectModel = dayModel;
              _isSelectCheckInTime = dayModel.isSelect ? true : false;
              _selectCheckInTime = '$year.$month.$checkInTime';
              _selectCheckInTimeModel = dayModel;
            }
          }
        }
        _checkInDays = 0;
        _isSelectLeaveTime = false;
        _selectLeaveTime = '选择离开时间';
        _isSelectLeaveTimeModel = null;
      } else if(_comparerTime(firstModel, year, month, checkInTime) == Location.mid){//点击了自己
        for(int i=0; i<_list.length; i++) {
          CalendarItemViewModel model = _list[i];
          model.lastSelectModel = null;
          if((model.month+model.year*100) == (month+year*100)){
            for(int i=0; i<model.list.length; i++) {
              DayModel dayModel = model.list[i];
              if(_comparerTime(dayModel, year, month, checkInTime) == Location.mid){
                dayModel.isSelect = !dayModel.isSelect;
                model.firstSelectModel = dayModel.isSelect ? dayModel : null;
                _selectCheckInTimeModel = dayModel.isSelect ? dayModel : null;
                _isSelectCheckInTime = dayModel.isSelect ? true : false;
                _selectCheckInTime = dayModel.isSelect ? '$year.$month.$checkInTime' : '选择入住时间';
              }
            }
          }
        }
        _checkInDays = 0;
        _isSelectLeaveTime = false;
        _selectLeaveTime = '选择离开时间';
        _isSelectLeaveTimeModel = null;
      } else if (_comparerTime(firstModel, year, month, checkInTime) == Location.right){
        if((month+year*100) == (firstModel.month+firstModel.year*100)){
          // 统计天数
          int _calculaterDays = 1;
          for(int i=0; i<_list.length; i++) {
            CalendarItemViewModel model = _list[i];
            if((model.month+model.year*100) == (month+year*100)){
              for(int i=0; i<model.list.length; i++) {
                DayModel dayModel = model.list[i];
                if(dayModel.dayNum == checkInTime) {
                  dayModel.isSelect = true;
                  model.lastSelectModel = dayModel;
                  _isSelectLeaveTimeModel = dayModel;
                  _isSelectLeaveTime = true;
                  _selectLeaveTime = '$year.$month.$checkInTime';
                }else if(dayModel.dayNum > firstModel.dayNum && dayModel.dayNum<checkInTime){
                  dayModel.isSelect = true;
                  _calculaterDays++;
                }
              }
            }
          }
          _checkInDays = _calculaterDays;

          ///--------------------------------------------
          if(_checkInDays > 29){
            Scaffold.of(context)
                .showSnackBar(SnackBar(content: Text('日期间隔不能超过30天')));
            _checkInDays = 0;
            _isSelectLeaveTime = false;
            _selectLeaveTime = '选择离开时间';
            _isSelectLeaveTimeModel = null;


            for(int i=0; i<_list.length; i++) {
              CalendarItemViewModel model = _list[i];
              model.lastSelectModel = null;
              if((model.month+model.year*100) == (month+year*100)){
                for(int i=0; i<model.list.length; i++) {
                  DayModel dayModel = model.list[i];
                  if(dayModel.dayNum == checkInTime) {
                    dayModel.isSelect = false;
                  }else if(dayModel.dayNum > firstModel.dayNum && dayModel.dayNum<checkInTime){
                    dayModel.isSelect = false;
                  }
                }
              }
            }



          }

        } else {
          // 统计天数
          int _calculaterDays = 1;
          for(int i=0; i<_list.length; i++) {
            CalendarItemViewModel model = _list[i];
            //入住当月
            if((model.month+model.year*100) == (firstModel.month+firstModel.year*100)){
              for(int i=0; i<model.list.length; i++) {
                DayModel dayModel = model.list[i];
                if (dayModel.dayNum > firstModel.dayNum){
                  dayModel.isSelect = true;
                  _calculaterDays++;
                }
              }
            } else if((model.month + model.year*100) >(firstModel.month+firstModel.year*100) && (model.month + model.year*100)<(month+year*100)){
              for(int i=0; i<model.list.length; i++) {
                DayModel dayModel = model.list[i];
                dayModel.isSelect = true;
                _calculaterDays++;
              }
            } else if((month+year*100) == (model.month+model.year*100)){
              for(int i=0; i<model.list.length; i++) {
                DayModel dayModel = model.list[i];
                if(dayModel.dayNum < checkInTime){
                  dayModel.isSelect = true;
                  _calculaterDays++;
                } else if (dayModel.dayNum == checkInTime) {
                  dayModel.isSelect = true;
                  model.lastSelectModel = dayModel;
                  _isSelectLeaveTimeModel = dayModel;
                  _isSelectLeaveTime = true;
                  _selectLeaveTime = '$year.$month.$checkInTime';
                }
              }
            }
          }
          _checkInDays = _calculaterDays;

          ///--------------------------------------------
          if(_checkInDays > 29){
            Scaffold.of(context)
                .showSnackBar(SnackBar(content: Text('日期间隔不能超过30天')));
            _checkInDays = 0;
            _isSelectLeaveTime = false;
            _selectLeaveTime = '选择离开时间';
            _isSelectLeaveTimeModel = null;


            for(int i=0; i<_list.length; i++) {
              CalendarItemViewModel model = _list[i];
              model.lastSelectModel = null;
              if((model.month+model.year*100) == (firstModel.month+firstModel.year*100)){
                for(int i=0; i<model.list.length; i++) {
                  DayModel dayModel = model.list[i];
                  if (dayModel.dayNum > firstModel.dayNum){
                    dayModel.isSelect = false;
                  }
                }
              } else if((model.month + model.year*100) >(firstModel.month+firstModel.year*100) && (model.month + model.year*100)<(month+year*100)){
                for(int i=0; i<model.list.length; i++) {
                  DayModel dayModel = model.list[i];
                  dayModel.isSelect = false;
                }
              } else if((month+year*100) == (model.month+model.year*100)){
                for(int i=0; i<model.list.length; i++) {
                  DayModel dayModel = model.list[i];
                  if(dayModel.dayNum < checkInTime){
                    dayModel.isSelect = false;
                  } else if (dayModel.dayNum == checkInTime) {
                    dayModel.isSelect = false;
                  }
                }
              }


            }


          }
        }
      }
    } else if(firstModel == null && lastModel == null){
      for(int i=0; i<_list.length; i++) {
        CalendarItemViewModel model = _list[i];
        model.firstSelectModel = null;
        model.lastSelectModel = null;
        firstModel = null;
        lastModel = null;
        for(int i=0; i<model.list.length; i++) {
          DayModel dayModel = model.list[i];
          dayModel.isSelect = false;
          if(_comparerTime(dayModel, year, month, checkInTime) == Location.mid){
            dayModel.isSelect = true;
            model.firstSelectModel = dayModel;
            _isSelectCheckInTime = true;
            _selectCheckInTimeModel = dayModel;
            _isSelectLeaveTime = false;
            _selectCheckInTime = '$year.$month.$checkInTime';
          }
        }
      }
    }
  }

  /*
  * 点击日期的回调事件
  * */
  _updateCheckInLeaveTime(int year, int month, int checkInTime) {
    // 更新数据源
    _updateDataSource(year, month, checkInTime);
    // 刷新UI
    setState(() {});
  }

  /*
  * 底部确定按钮的点击事件
  * */
  _sureButtonTap() {
    if(!_isSelectCheckInTime){
      Scaffold.of(context)
          .showSnackBar(SnackBar(content: Text('请选择开始时间')));
      return;
    } else if (!_isSelectLeaveTime){
      Scaffold.of(context)
          .showSnackBar(SnackBar(content: Text('请选择结束时间')));
      return;
    }
    print('${_selectCheckInTimeModel.year},${_selectCheckInTimeModel.month},${_selectCheckInTimeModel.dayNum}');
    print('${_isSelectLeaveTimeModel.year},${_isSelectLeaveTimeModel.month},${_isSelectLeaveTimeModel.dayNum}');
    print('开始日期：$_selectCheckInTime, 结束时间：$_selectLeaveTime, 共$_checkInDays天');
    // 把日期回调给外部
    widget.selectDateOnTap(_selectCheckInTimeModel,_isSelectLeaveTimeModel);
    Navigator.pop(context);
  }
}