import 'package:flutter/material.dart';
import 'package:flutter_choose_date/chooseDate/time_utils.dart';

import 'calendar_page_viewModel.dart';



typedef void OnTapDayItem(int year, int month, int checkInTime);

class CalendarItem extends StatefulWidget {
  final CalendarItemViewModel itemModel;
  final OnTapDayItem dayItemOnTap;

  CalendarItem(this.dayItemOnTap, this.itemModel);

  @override
  _CalendarItemState createState() => _CalendarItemState();
}

class _CalendarItemState extends State<CalendarItem> {
  // 日历显示几行
  int _rows = 0;
  List<DayModel> _listModel = <DayModel>[];

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    _listModel = widget.itemModel.list;
  }

  @override
  Widget build(BuildContext context) {
    double screenWith = MediaQuery.of(context).size.width;
    // 显示几行
    _rows = TimeUtil.getRowsForMonthYear(widget.itemModel.year,
        widget.itemModel.month, MaterialLocalizations.of(context));

    return Container(
      width: screenWith,
      height: 25.0 + _rows * 54.0 + 20.0 + 20.0,
      child: Column(
        children: <Widget>[
          SizedBox(
            height: 20,
          ),
          _yearMonthItem(widget.itemModel.year, widget.itemModel.month), //25
          SizedBox(
            height: 20,
          ),
          _monthAllDays(widget.itemModel.year, widget.itemModel.month, context),
        ],
      ),
    );
  }

  /*
  * 显示年月的组件，需要传入年月日期
  * */
  _yearMonthItem(int year, int month) {
    return Container(
      alignment: Alignment.center,
      height: 25,
      child: Text(
        '$year.$month',
        textAlign: TextAlign.center,
        style: TextStyle(
          color: Color(0xFF212121),
          fontSize: 18,
          fontFamily: 'Avenir-Heavy',
        ),
      ),
    );
  }

  _monthAllDays(int year, int month, BuildContext context) {
    double screenWith = MediaQuery.of(context).size.width;

    // 当前月前面空的天数
    int emptyDays = TimeUtil.numberOfHeadPlaceholderForMonth(
        year, month, MaterialLocalizations.of(context));

    List<Widget> _list = <Widget>[];

    for (int i = 1; i <= emptyDays; i++) {
      _list.add(_dayEmptyTitleItem(context));
    }

    for (int i = 1; i <= _listModel.length; i++) {
      _list.add(_dayTitleItem(_listModel[i - 1], context));
    }

    List<Row> _rowList = <Row>[
      Row(
        children: _list.sublist(0, 7),
      ),
      Row(
        children: _list.sublist(7, 14),
      ),
      Row(
        children: _list.sublist(14, 21),
      ),
    ];

    if (_rows == 4) {
      _rowList.add(
        Row(
          children: _list.sublist(21, _list.length),
        ),
      );
    } else if (_rows == 5) {
      _rowList.add(
        Row(
          children: _list.sublist(21, 28),
        ),
      );
      _rowList.add(
        Row(
          children: _list.sublist(28, _list.length),
        ),
      );
    } else if (_rows == 6) {
      _rowList.add(
        Row(
          children: _list.sublist(21, 28),
        ),
      );
      _rowList.add(
        Row(
          children: _list.sublist(28, 35),
        ),
      );
      _rowList.add(
        Row(
          children: _list.sublist(35, _list.length),
        ),
      );
    }
    return Container(
      width: screenWith - 40,
      color: Colors.white,
      height: 54.0 * _rows,
      child: Column(
        children: _rowList,
      ),
    );
  }

  /*
  * number 月的几号
  * isOverdue 是否过期
  * */
  _dayTitleItem(DayModel model, BuildContext context) {
    double screenWith = MediaQuery.of(context).size.width;
    double singleW = (screenWith - 40) / 7;
    String dayTitle = model.day;
    if (widget.itemModel.firstSelectModel != null &&
        model.isSelect &&
        model.dayNum == widget.itemModel.firstSelectModel.dayNum) {
      dayTitle = '开始';
    }
    if (widget.itemModel.lastSelectModel != null &&
        model.isSelect &&
        model.dayNum == widget.itemModel.lastSelectModel.dayNum) {
      dayTitle = '结束';
    }
    return GestureDetector(
      onTap: () {
        if(model.isOverdue) return;
        _dayTitleItemTap(model);
      },
      child: Container(
        margin: EdgeInsets.all(0),
        color: Color(0x00000000),
        child: Stack(
          children: <Widget>[
            Positioned(
              left: singleW/2-12,
              right: singleW/2-12,
              bottom: 15,
              child: Visibility(
                  visible: model.isOverdue ? false : model.isSelect,
                  child: Container(
                    height: 24,
                    width: 24,
                    decoration: BoxDecoration(
                      color: Color(0xFF4980F9),
                      borderRadius: BorderRadius.all(Radius.circular(12))
                    ),
                  )),
            ),
            Positioned(
              left: 0,
              right: 0,
              bottom: 0,
              child: Visibility(
                  visible: model.isOverdue ? false : model.isSelect,
                  child: Container(
                    child: Text(
                      (dayTitle=="结束" || dayTitle=="开始")?dayTitle:"",
                      textAlign: TextAlign.center,
                      style: TextStyle(color: Color(0xFF5797a0),
                          fontSize: 10.5),),
                  )),
            ),
            Container(
              width: singleW,
              height: 54,
              alignment: Alignment.center,
              child: Text(
                model.day,
                textAlign: TextAlign.center,
                style: TextStyle(
                  color: model.isOverdue
                      ? Color(0xFFBDBDBD)
                      : (model.isSelect? Colors.white: Color(0xFF212121)),
                  fontSize: 14,
                  fontFamily: 'Avenir-Medium',
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  _dayEmptyTitleItem(BuildContext context) {
    double screenWith = MediaQuery.of(context).size.width;
    double singleW = (screenWith - 40) / 7;
    return Container(
      width: singleW,
      height: 52,
    );
  }

  _dayTitleItemTap(DayModel model) {
    widget.dayItemOnTap(
        widget.itemModel.year, widget.itemModel.month, model.dayNum);
    setState(() {});
  }
}
